#!/bin/bash

# Initial packager: Filippo "NeverBorn" Mesirca (filippo.mesirca@gmail.com)
# Maintainer : Son|c (emmanuel.bailleul@cegetel.net)
# With help from: Polesz
# Sources location : 
# http://mirrors.ircam.fr/pub/mysql/Downloads/MySQL-5.0/mysql-5.0.75.tar.gz

# program name/version
softversion='5.0.75'
softname1='mysql'
softname2='libmysqlclient'

# architecture/package1 version
arch='i486'
cpu='i686'
pkgversion='54.1'

# number of parallel tasks of make
ntasks='-j2'

# parameters needed for the build process
buildir=$(pwd)
package1="$softname1-$softversion-$arch-$pkgversion"
package2="$softname2-$softversion-$arch-$pkgversion"

# prepare the build result directory
dest1="$buildir/$package1"
dest2="$buildir/$package2"
rm -rf $dest1 $dest2
rm -f $package1.* $package2.*
mkdir -p $dest1
mkdir -p $dest2
mkdir -p $dest1/install
mkdir -p $dest2/install
mkdir -p $dest1/usr/src/$softname1-$softversion
mkdir -p $dest2/usr/src/$softname2-$softversion
mkdir -p $dest2/usr/man/man1
mkdir -p $dest2/usr/bin

# make a slack-desc file
cat <<EOF > $dest1/install/slack-desc
     |-----handy-ruler------------------------------------------------------|
mysql: MySQL (Database management system)
mysql:
mysql: MySQL is a fast, multi-threaded, multi-user and robust SQL
mysql: (Structured Query Language) DBMS (DataBase Management System). It
mysql: comes with a nice API which makes it easy to integrate into other
mysql: applications.
mysql:
mysql:
mysql:
mysql:
mysql:
EOF

cat <<EOF > $dest2/install/slack-desc
     |-----handy-ruler------------------------------------------------------|
libmysqlclient: MySQL libs and include files
libmysqlclient:
libmysqlclient: MySQL is a fast, multi-threaded, multi-user and 
libmysqlclient: robust SQL (Structured Query Language) DBMS 
libmysqlclient: (DataBase Management System). It comes with a
libmysqlclient: nice API which makes it easy to integrate into
libmysqlclient: other applications.
libmysqlclient:
libmysqlclient:
libmysqlclient:
libmysqlclient:
EOF

# extract sources
srcdir="mysql-$softversion"
cd $buildir
rm -rf $srcdir
tar zxvf $srcdir.tar.gz
cd $srcdir

# set correct permissions
chown -R root.root . 
find . -perm 777 -exec chmod 755 {} \;
find . -perm 775 -exec chmod 755 {} \;
find . -perm 666 -exec chmod 644 {} \;
find . -perm 664 -exec chmod 644 {} \;
 
# configure package1 for compilation
export CFLAGS="-O2 -march=$arch -mtune=$cpu"
export CXXFLAGS="-O2 -march=$arch -mtune=$cpu"
./configure \
   --prefix=/usr \
   --localstatedir=/var/lib/mysql \
   --enable-assembler \
   --enable-thread-safe-client \
   --with-mysqld-user=mysql \
   --without-debug \
   --with-openssl=/usr \
   --with-extra-charsets=all \
   --with-unix-socket-path=/var/run/mysql/mysql.sock \
   --with-big-tables \
   --without-bench \
   --program-prefix="" \
   --program-suffix=""
   
# compile package1
make $ntasks

# install package1 to the target directory
make install DESTDIR=$dest1

# copy the support files: it will be up to the user to decide which
# configuration to use
mkdir -p $dest1/etc
cd $buildir/$srcdir/support-files
cp -a my-huge.cnf.sh my-large.cnf.sh my-medium.cnf.sh my-small.cnf.sh $dest1/etc
#zcat $buildir/my.cnf.gz > $dest1/etc/my.cnf.new

echo "# Example MySQL config file for medium systems.
#
# This is for a system with little memory (32M - 64M) where MySQL plays
# an important part, or systems up to 128M where MySQL is used together with
# other programs (such as a web server)
#
# You can copy this file to
# /etc/my.cnf to set global options,
# mysql-data-dir/my.cnf to set server-specific options (in this
# installation this directory is @localstatedir@) or
# ~/.my.cnf to set user-specific options.
#
# In this file, you can use all long options that a program supports.
# If you want to know which options a program supports, run the program
# with the \"--help\" option.

# The following options will be passed to all MySQL clients
[client]
#password	= your_password
port		= 3306
socket		= /var/run/mysql/mysql.sock

# Here follows entries for some specific programs

# The MySQL server
[mysqld]
port		= 3306
socket		= /var/run/mysql/mysql.sock
skip-locking
key_buffer = 16M
max_allowed_packet = 1M
table_cache = 64
sort_buffer_size = 512K
net_buffer_length = 8K
read_buffer_size = 256K
read_rnd_buffer_size = 512K
myisam_sort_buffer_size = 8M

# Don't listen on a TCP/IP port at all. This can be a security enhancement,
# if all processes that need to connect to mysqld run on the same host.
# All interaction with mysqld must be made via Unix sockets or named pipes.
# Note that using this option without enabling named pipes on Windows
# (via the \"enable-named-pipe\" option) will render mysqld useless!
# 
skip-networking

# Replication Master Server (default)
# binary logging is required for replication
log-bin=mysql-bin

# required unique id between 1 and 2^32 - 1
# defaults to 1 if master-host is not set
# but will not function as a master if omitted
server-id	= 1

# Replication Slave (comment out master section to use this)
#
# To configure this host as a replication slave, you can choose between
# two methods :
#
# 1) Use the CHANGE MASTER TO command (fully described in our manual) -
#    the syntax is:
#
#    CHANGE MASTER TO MASTER_HOST=<host>, MASTER_PORT=<port>,
#    MASTER_USER=<user>, MASTER_PASSWORD=<password> ;
#
#    where you replace <host>, <user>, <password> by quoted strings and
#    <port> by the master's port number (3306 by default).
#
#    Example:
#
#    CHANGE MASTER TO MASTER_HOST='125.564.12.1', MASTER_PORT=3306,
#    MASTER_USER='joe', MASTER_PASSWORD='secret';
#
# OR
#
# 2) Set the variables below. However, in case you choose this method, then
#    start replication for the first time (even unsuccessfully, for example
#    if you mistyped the password in master-password and the slave fails to
#    connect), the slave will create a master.info file, and any later
#    change in this file to the variables' values below will be ignored and
#    overridden by the content of the master.info file, unless you shutdown
#    the slave server, delete master.info and restart the slaver server.
#    For that reason, you may want to leave the lines below untouched
#    (commented) and instead use CHANGE MASTER TO (see above)
#
# required unique id between 2 and 2^32 - 1
# (and different from the master)
# defaults to 2 if master-host is set
# but will not function as a slave if omitted
#server-id       = 2
#
# The replication master for this slave - required
#master-host     =   <hostname>
#
# The username the slave will use for authentication when connecting
# to the master - required
#master-user     =   <username>
#
# The password the slave will authenticate with when connecting to
# the master - required
#master-password =   <password>
#
# The port the master is listening on.
# optional - defaults to 3306
#master-port     =  <port>
#
# binary logging - not required for slaves, but recommended
#log-bin=mysql-bin

# Point the following paths to different dedicated disks
#tmpdir		= /tmp/		
#log-update 	= /path-to-dedicated-directory/hostname

# Uncomment the following if you are using BDB tables
#bdb_cache_size = 4M
#bdb_max_lock = 10000

# Uncomment the following if you are using InnoDB tables
#innodb_data_home_dir = @localstatedir@/
#innodb_data_file_path = ibdata1:10M:autoextend
#innodb_log_group_home_dir = @localstatedir@/
#innodb_log_arch_dir = @localstatedir@/
# You can set .._buffer_pool_size up to 50 - 80 %
# of RAM but beware of setting memory usage too high
#innodb_buffer_pool_size = 16M
#innodb_additional_mem_pool_size = 2M
# Set .._log_file_size to 25 % of buffer pool size
#innodb_log_file_size = 5M
#innodb_log_buffer_size = 8M
#innodb_flush_log_at_trx_commit = 1
#innodb_lock_wait_timeout = 50

[mysqldump]
quick
max_allowed_packet = 16M

[mysql]
no-auto-rehash
# Remove the next comment character if you are not familiar with SQL
#safe-updates

[isamchk]
key_buffer = 20M
sort_buffer_size = 20M
read_buffer = 2M
write_buffer = 2M

[myisamchk]
key_buffer = 20M
sort_buffer_size = 20M
read_buffer = 2M
write_buffer = 2M

[mysqlhotcopy]
interactive-timeout
" > $dest1/etc/my.cnf.new

# copy the doc files
mkdir -p $dest1/usr/doc/$softname1-$softversion
cd $buildir/$srcdir
cp -a \
   COPYING ChangeLog EXCEPTIONS-CLIENT INSTALL-SOURCE README Docs/INSTALL-BINARY \
   $dest1/usr/doc/$softname1-$softversion

# make the folder where the databases are stored
mkdir -p $dest1/var/lib/mysql
chown mysql.mysql $dest1/var/lib/mysql
chmod 750 $dest1/var/lib/mysql

# make the folder where the socket file is stored
mkdir -p $dest1/var/run/mysql
chown mysql.mysql $dest1/var/run/mysql
chmod 755 $dest1/var/run/mysql

# put the initialization script in $dest/etc/rc.d/ (rc.mysqld taken from
# Slackware-10.2)
mkdir -p $dest1/etc/rc.d
#zcat $buildir/rc.mysqld.gz > $dest1/etc/rc.d/rc.mysqld.new

echo "#!/bin/sh
# Start/stop/restart mysqld.
#
# Copyright 2003 Patrick J. Volkerding, Concord, CA
# Copyright 2003 Slackware Linux, Inc., Concord, CA
#
# Modified by _PN_Boy_
#
# This program comes with NO WARRANTY, to the extent permitted by law.
# You may redistribute copies of this program under the terms of the
# GNU General Public License.

# To start MySQL automatically at boot, be sure this script is executable:
# chmod 755 /etc/rc.d/rc.mysqld

# Before you can run MySQL, you must have a database.  To install an initial
# database, do this as root:
#
#   su - mysql
#   mysql_install_db
#   exit
#   /usr/bin/mysqladmin -u root password 'new-password'
#
# Note that step one is becoming the mysql user.  It's important to do this
# before making any changes to the database, or mysqld won't be able to write
# to it later (this can be fixed with 'chown -R mysql.mysql /var/lib/mysql').

# To allow outside connections to the database comment out the next line.
# If you don't need incoming network connections, then leave the line
# uncommented to improve system security.
SKIP=\"--skip-networking\"
hostname=\`/bin/hostname\`

# Start mysqld:
mysqld_start() {
  if [ -x /usr/bin/mysqld_safe ]; then
    # If there is an old PID file (no mysqld running), clean it up:
    if [ -r /var/run/mysql/mysql.pid ]; then
      if ! ps axc | grep mysqld 1> /dev/null 2> /dev/null ; then
        echo \"Cleaning up old /var/run/mysql/mysql.pid.\"
        rm -f /var/run/mysql/mysql.pid
      fi
    fi
    if [ ! -d /var/lib/mysql/mysql ]; then
      su - mysql -c \"/usr/bin/mysql_install_db\"
      echo \"###############################################################\"
      echo \"PLEASE REMEMBER TO SET A PASSWORD FOR THE MySQL root USER !\"
      echo \"To do so, first start the server with :\"
      echo \"/usr/bin/mysqld_safe\"
      echo \"Then, issue the following commands :\"
      echo \"/usr/bin/mysqladmin -u root password 'new-password'\"
      echo \"/usr/bin/mysqladmin -u root -h \$hostname password 'new-password'\"
      echo \"Once this is done, you should restart your server with /etc/rc.d/rc.mysql restart\"
      echo \"See the manual for more instructions.\"
      echo \"###############################################################\"
    fi
    /usr/bin/mysqld_safe --datadir=/var/lib/mysql --pid-file=/var/run/mysql/mysql.pid $SKIP &
  fi
}

# Stop mysqld:
mysqld_stop() {
  # If there is no PID file, ignore this request...
  if [ -r /var/run/mysql/mysql.pid ]; then
    killall mysqld
    # Wait at least one minute for it to exit, as we don't know how big the DB is...
    for second in 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 \
      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 60 ; do
      if [ ! -r /var/run/mysql/mysql.pid ]; then
        break;
      fi
      sleep 1
    done
    if [ \"\$second\" = \"60\" ]; then
      echo \"WARNING:  Gave up waiting for mysqld to exit!\"
      sleep 15
    fi
  fi
}

# Restart mysqld:
mysqld_restart() {
  mysqld_stop
  mysqld_start
}

case \"\$1\" in
'start')
  mysqld_start
  ;;
'stop')
  mysqld_stop
  ;;
'restart')
  mysqld_restart
  ;;
*)
  echo \"usage \$0 start|stop|restart\"
esac" > $dest1/etc/rc.d/rc.mysqld.new

chmod 644 $dest1/etc/rc.d/rc.mysqld.new

# remove test directory (it occupies 17MB)
rm -rf $dest1/usr/mysql-test

# strip executables 
( cd $dest1
   find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
   find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
)
  
# gzip man pages and info files
gzip -9 $dest1/usr/man/*/*
gzip -9 $dest1/usr/info/*

# add some symbolic links (idea taken from Slackware-10.2 slackbuild)
cd $dest1/usr/lib
ln -sf mysql/libmysqlclient.so .
ln -sf mysql/libmysqlclient.so.15 .
ln -sf mysql/libmysqlclient_r.so .
ln -sf mysql/libmysqlclient_r.so.15 .

# put the ZenWalk build and other needed files in the right directory
cp -a $buildir/build-$softname1.sh $dest1/usr/src/$softname1-$softversion
cp -a $buildir/build-$softname1.sh $dest2/usr/src/$softname2-$softversion
#cp -a $buildir/rc.mysqld.gz $dest1/usr/src/$softname1-$softversion
#cp -a $buildir/doinst.sh.gz $dest1/usr/src/$softname1-$softversion
#cp -a $buildir/my.cnf.gz $dest1/usr/src/$softname1-$softversion

# change files ownerships for the $dest/usr/bin and $dest/usr/sbin
chown -R root.root $dest1/usr/bin

# make the package and move it to the build directory
#zcat $buildir/doinst.sh.gz > $dest1/install/doinst.sh

echo "# Install the info files for this package
if [ -x /usr/bin/install-info ]
then
   /usr/bin/install-info --info-dir=usr/info usr/info/mysql.info.gz 2>/dev/null
fi
config() {
  NEW=\"\$1\"
  OLD=\"\`dirname \$NEW\`/\`basename \$NEW .new\`\"
  # If there's no config file by that name, mv it over:
  if [ ! -r \$OLD ]; then
    mv \$NEW \$OLD
  elif [ \"\`cat \$OLD | md5sum\`\" = \"\`cat \$NEW | md5sum\`\" ]; then # toss the redundant copy
    rm \$NEW
  fi
  # Otherwise, we leave the .new copy for the admin to consider...
}
config etc/rc.d/rc.mysqld.new
config etc/my.cnf.new" > $dest1/install/doinst.sh

# Split the packages
cd $dest1
mv usr/include usr/lib $dest2/usr/
mv usr/bin/mysql_config $dest2/usr/bin
mv usr/man/man1/mysql_config.1.gz $dest2/usr/man/man1/

#------------------( Create The *.dep Files )
cd $dest1
requiredbuilder -z $dest1 > $buildir/$package1.dep
echo ",libmysqlclient" >> $buildir/$package1.dep

cd $dest2
requiredbuilder -z $dest2 > $buildir/$package2.dep

cd $dest1
makepkg -l y -c n $package1.tgz
mv $package1.tgz $buildir
cd $dest2
makepkg -l y -c n $package2.tgz
mv $package2.tgz $buildir

#------------------( Generate the MD5 Check Sum )
cd $buildir
rm -f $package1.md5
md5sum $package1.tgz > $package1.md5
rm -f $package2.md5
md5sum $package2.tgz > $package2.md5

# Create source files
echo "http://mirrors.ircam.fr/pub/mysql/Downloads/MySQL-5.0/mysql-5.0.75.tar.gz" > $package1.src
echo "http://download.zenwalk.org/people/emmanuel/$softname1-$softversion/build-$softname1.sh" >> $package1.src

cp $package1.src $package2.src

if [ "$1" == "--cleanup" ]; then
  rm -rf $dest1 $dest2
  rm -rf $srcdir
fi

